@@ -1,19 +1,22 @@
-use std::{collections::BTreeMap, convert::TryFrom};
+use std::collections::BTreeMap;
 
 use ruma::{
-    api::client::r0::{
-        push::get_notifications::Notification,
-        sync::sync_events::{
+    api::client::{
+        push::get_notifications::v3::Notification,
+        sync::sync_events::v3::{
             DeviceLists, Ephemeral, GlobalAccountData, InvitedRoom, Presence, RoomAccountData,
             State, ToDevice, UnreadNotificationsCount as RumaUnreadNotificationsCount,
         },
     },
     events::{
-        room::member::MemberEventContent, AnySyncRoomEvent, StateEvent, StrippedStateEvent,
-        SyncStateEvent, Unsigned,
+        room::member::{
+            MembershipState, RoomMemberEvent, RoomMemberEventContent, StrippedRoomMemberEvent,
+            SyncRoomMemberEvent,
+        },
+        AnyRoomEvent, AnySyncRoomEvent,
     },
     serde::Raw,
-    DeviceIdBox, DeviceKeyAlgorithm, EventId, MilliSecondsSinceUnixEpoch, RoomId, UserId,
+    DeviceKeyAlgorithm, OwnedDeviceId, OwnedEventId, OwnedRoomId, OwnedUserId, UserId,
 };
 use serde::{Deserialize, Serialize};
 
@@ -25,9 +28,9 @@ pub struct AmbiguityChange {
     /// event itself ambiguous because of the event.
     pub member_ambiguous: bool,
     /// Has another user been disambiguated because of this event.
-    pub disambiguated_member: Option<UserId>,
+    pub disambiguated_member: Option<OwnedUserId>,
     /// Has another user become ambiguous because of this event.
-    pub ambiguated_member: Option<UserId>,
+    pub ambiguated_member: Option<OwnedUserId>,
 }
 
 /// Collection of ambiguioty changes that room member events trigger.
@@ -35,7 +38,7 @@ pub struct AmbiguityChange {
 pub struct AmbiguityChanges {
     /// A map from room id to a map of an event id to the `AmbiguityChange` that
     /// the event with the given id caused.
-    pub changes: BTreeMap<RoomId, BTreeMap<EventId, AmbiguityChange>>,
+    pub changes: BTreeMap<OwnedRoomId, BTreeMap<OwnedEventId, AmbiguityChange>>,
 }
 
 /// The verification state of the device that sent an event to us.
@@ -72,10 +75,10 @@ pub enum AlgorithmInfo {
 pub struct EncryptionInfo {
     /// The user ID of the event sender, note this is untrusted data unless the
     /// `verification_state` is as well trusted.
-    pub sender: UserId,
+    pub sender: OwnedUserId,
     /// The device ID of the device that sent us the event, note this is
     /// untrusted data unless `verification_state` is as well trusted.
-    pub sender_device: DeviceIdBox,
+    pub sender_device: OwnedDeviceId,
     /// Information about the algorithm that was used to encrypt the event.
     pub algorithm_info: AlgorithmInfo,
     /// The verification state of the device that sent us the event, note this
@@ -95,12 +98,29 @@ pub struct SyncRoomEvent {
     pub encryption_info: Option<EncryptionInfo>,
 }
 
+impl SyncRoomEvent {
+    /// Get the event id of this `SyncRoomEvent` if the event has any valid id.
+    pub fn event_id(&self) -> Option<OwnedEventId> {
+        self.event.get_field::<OwnedEventId>("event_id").ok().flatten()
+    }
+}
+
 impl From<Raw<AnySyncRoomEvent>> for SyncRoomEvent {
     fn from(inner: Raw<AnySyncRoomEvent>) -> Self {
         Self { encryption_info: None, event: inner }
     }
 }
 
+impl From<RoomEvent> for SyncRoomEvent {
+    fn from(o: RoomEvent) -> Self {
+        // This conversion is unproblematic since a SyncRoomEvent is just a
+        // RoomEvent without the room_id. By converting the raw value in this
+        // way, we simply cause the `room_id` field in the json to be ignored by
+        // a subsequent deserialization.
+        Self { encryption_info: o.encryption_info, event: o.event.cast() }
+    }
+}
+
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct SyncResponse {
     /// The batch token to supply in the `since` param of the next `/sync`
@@ -124,7 +144,7 @@ pub struct SyncResponse {
     /// Collection of ambiguity changes that room member events trigger.
     pub ambiguity_changes: AmbiguityChanges,
     /// New notifications per room.
-    pub notifications: BTreeMap<RoomId, Vec<Notification>>,
+    pub notifications: BTreeMap<OwnedRoomId, Vec<Notification>>,
 }
 
 impl SyncResponse {
@@ -136,7 +156,7 @@ impl SyncResponse {
 #[derive(Clone, Debug)]
 pub struct RoomEvent {
     /// The actual event.
-    pub event: Raw<ruma::events::AnyRoomEvent>,
+    pub event: Raw<AnyRoomEvent>,
     /// The encryption info about the event. Will be `None` if the event was not
     /// encrypted.
     pub encryption_info: Option<EncryptionInfo>,
@@ -145,11 +165,11 @@ pub struct RoomEvent {
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct Rooms {
     /// The rooms that the user has left or been banned from.
-    pub leave: BTreeMap<RoomId, LeftRoom>,
+    pub leave: BTreeMap<OwnedRoomId, LeftRoom>,
     /// The rooms that the user has joined.
-    pub join: BTreeMap<RoomId, JoinedRoom>,
+    pub join: BTreeMap<OwnedRoomId, JoinedRoom>,
     /// The rooms that the user has been invited to.
-    pub invite: BTreeMap<RoomId, InvitedRoom>,
+    pub invite: BTreeMap<OwnedRoomId, InvitedRoom>,
 }
 
 /// Updates to joined rooms.
@@ -243,96 +263,69 @@ impl Timeline {
     }
 }
 
+/// A slice of the timeline in the room.
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
+pub struct TimelineSlice {
+    /// The `next_batch` or `from` token used to obtain this slice
+    pub start: String,
+
+    /// The `prev_batch` or `to` token used to obtain this slice
+    /// If `None` this `TimelineSlice` is the beginning of the room
+    pub end: Option<String>,
+
+    /// Whether the number of events returned for this slice was limited
+    /// by a `limit`-filter when requesting
+    pub limited: bool,
+
+    /// A list of events.
+    pub events: Vec<SyncRoomEvent>,
+
+    /// Whether this is a timeline slice obtained from a `SyncResponse`
+    pub sync: bool,
+}
+
+impl TimelineSlice {
+    pub fn new(
+        events: Vec<SyncRoomEvent>,
+        start: String,
+        end: Option<String>,
+        limited: bool,
+        sync: bool,
+    ) -> Self {
+        Self { start, end, events, limited, sync }
+    }
+}
+
+/// Wrapper around both MemberEvent-Types
+#[allow(clippy::large_enum_variant)]
 #[derive(Clone, Debug, Deserialize, Serialize)]
-#[serde(
-    try_from = "SyncStateEvent<MemberEventContent>",
-    into = "SyncStateEvent<MemberEventContent>"
-)]
-pub struct MemberEvent {
-    pub content: MemberEventContent,
-    pub event_id: EventId,
-    pub origin_server_ts: MilliSecondsSinceUnixEpoch,
-    pub prev_content: Option<MemberEventContent>,
-    pub sender: UserId,
-    pub state_key: UserId,
-    pub unsigned: Unsigned,
+pub enum MemberEvent {
+    Sync(SyncRoomMemberEvent),
+    Stripped(StrippedRoomMemberEvent),
 }
 
-impl TryFrom<SyncStateEvent<MemberEventContent>> for MemberEvent {
-    type Error = ruma::identifiers::Error;
-
-    fn try_from(event: SyncStateEvent<MemberEventContent>) -> Result<Self, Self::Error> {
-        Ok(MemberEvent {
-            content: event.content,
-            event_id: event.event_id,
-            origin_server_ts: event.origin_server_ts,
-            prev_content: event.prev_content,
-            sender: event.sender,
-            state_key: UserId::try_from(event.state_key)?,
-            unsigned: event.unsigned,
-        })
-    }
-}
-
-impl TryFrom<StateEvent<MemberEventContent>> for MemberEvent {
-    type Error = ruma::identifiers::Error;
-
-    fn try_from(event: StateEvent<MemberEventContent>) -> Result<Self, Self::Error> {
-        Ok(MemberEvent {
-            content: event.content,
-            event_id: event.event_id,
-            origin_server_ts: event.origin_server_ts,
-            prev_content: event.prev_content,
-            sender: event.sender,
-            state_key: UserId::try_from(event.state_key)?,
-            unsigned: event.unsigned,
-        })
-    }
-}
-
-impl From<MemberEvent> for SyncStateEvent<MemberEventContent> {
-    fn from(other: MemberEvent) -> SyncStateEvent<MemberEventContent> {
-        SyncStateEvent {
-            content: other.content,
-            event_id: other.event_id,
-            sender: other.sender,
-            origin_server_ts: other.origin_server_ts,
-            state_key: other.state_key.to_string(),
-            prev_content: other.prev_content,
-            unsigned: other.unsigned,
+impl MemberEvent {
+    /// The inner Content of the wrapped Event
+    pub fn original_content(&self) -> Option<&RoomMemberEventContent> {
+        match self {
+            MemberEvent::Sync(e) => e.as_original().map(|e| &e.content),
+            MemberEvent::Stripped(e) => Some(&e.content),
         }
     }
-}
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
-#[serde(
-    try_from = "StrippedStateEvent<MemberEventContent>",
-    into = "StrippedStateEvent<MemberEventContent>"
-)]
-pub struct StrippedMemberEvent {
-    pub content: MemberEventContent,
-    pub sender: UserId,
-    pub state_key: UserId,
-}
-
-impl TryFrom<StrippedStateEvent<MemberEventContent>> for StrippedMemberEvent {
-    type Error = ruma::identifiers::Error;
-
-    fn try_from(event: StrippedStateEvent<MemberEventContent>) -> Result<Self, Self::Error> {
-        Ok(StrippedMemberEvent {
-            content: event.content,
-            sender: event.sender,
-            state_key: UserId::try_from(event.state_key)?,
-        })
+    /// The membership state of the user
+    pub fn membership(&self) -> &MembershipState {
+        match self {
+            MemberEvent::Sync(e) => e.membership(),
+            MemberEvent::Stripped(e) => &e.content.membership,
+        }
     }
-}
 
-impl From<StrippedMemberEvent> for StrippedStateEvent<MemberEventContent> {
-    fn from(other: StrippedMemberEvent) -> Self {
-        Self {
-            content: other.content,
-            sender: other.sender,
-            state_key: other.state_key.to_string(),
+    /// The user id associated to this member event
+    pub fn user_id(&self) -> &UserId {
+        match self {
+            MemberEvent::Sync(e) => e.state_key(),
+            MemberEvent::Stripped(e) => &e.state_key,
         }
     }
 }
@@ -343,7 +336,54 @@ impl From<StrippedMemberEvent> for StrippedStateEvent<MemberEventContent> {
 #[derive(Clone, Debug, Default, Deserialize, Serialize)]
 pub struct MembersResponse {
     /// The list of members events.
-    pub chunk: Vec<MemberEvent>,
+    pub chunk: Vec<RoomMemberEvent>,
     /// Collection of ambiguity changes that room member events trigger.
     pub ambiguity_changes: AmbiguityChanges,
 }
+
+#[cfg(test)]
+mod tests {
+    use ruma::{
+        event_id,
+        events::{
+            room::message::RoomMessageEventContent, AnySyncMessageLikeEvent, AnySyncRoomEvent,
+            MessageLikeUnsigned, OriginalMessageLikeEvent, SyncMessageLikeEvent,
+        },
+        room_id,
+        serde::Raw,
+        user_id, MilliSecondsSinceUnixEpoch,
+    };
+
+    use super::{RoomEvent, SyncRoomEvent};
+
+    #[test]
+    fn room_event_to_sync_room_event() {
+        let content = RoomMessageEventContent::text_plain("foobar");
+
+        let event = OriginalMessageLikeEvent {
+            content,
+            event_id: event_id!("$xxxxx:example.org").to_owned(),
+            room_id: room_id!("!someroom:example.com").to_owned(),
+            origin_server_ts: MilliSecondsSinceUnixEpoch::now(),
+            sender: user_id!("@carl:example.com").to_owned(),
+            unsigned: MessageLikeUnsigned::default(),
+        };
+
+        let room_event =
+            RoomEvent { event: Raw::new(&event).unwrap().cast(), encryption_info: None };
+
+        let converted_room_event: SyncRoomEvent = room_event.into();
+
+        let converted_event: AnySyncRoomEvent = converted_room_event.event.deserialize().unwrap();
+
+        let sync_event = AnySyncRoomEvent::MessageLike(AnySyncMessageLikeEvent::RoomMessage(
+            SyncMessageLikeEvent::Original(event.into()),
+        ));
+
+        // There is no PartialEq implementation for AnySyncRoomEvent, so we
+        // just compare a couple of fields here. The important thing is that
+        // the deserialization above worked.
+        assert_eq!(converted_event.event_id(), sync_event.event_id());
+        assert_eq!(converted_event.sender(), sync_event.sender());
+    }
+}
