@@ -1,16 +1,7 @@
 #![doc = include_str!("../README.md")]
-#![deny(
-    missing_debug_implementations,
-    trivial_casts,
-    trivial_numeric_casts,
-    unused_extern_crates,
-    unused_import_braces,
-    unused_qualifications
-)]
+#![warn(missing_debug_implementations)]
 
-pub use async_trait::async_trait;
 pub use instant;
-pub use uuid;
 
 pub mod deserialized_responses;
 pub mod executor;
@@ -28,6 +19,6 @@ impl<T: std::fmt::Debug + Send + Sync> AsyncTraitDeps for T {}
 /// it's used on WASM. WASM targets will not require `Send` and `Sync` to have
 /// implemented, while other targets will.
 #[cfg(target_arch = "wasm32")]
-pub trait AsyncTraitDeps: std::fmt::Debug + Send + Sync {}
+pub trait AsyncTraitDeps: std::fmt::Debug {}
 #[cfg(target_arch = "wasm32")]
-impl<T: std::fmt::Debug + Send + Sync> AsyncTraitDeps for T {}
+impl<T: std::fmt::Debug> AsyncTraitDeps for T {}
